@@ -64,6 +64,7 @@ import itertools
 import contextlib
 
 import six
+import jaraco.functools
 from jaraco.itertools import always_iterable, infinite_call
 from jaraco.functools import Throttler
 from jaraco.stream import buffer
@@ -76,18 +77,19 @@ except ImportError:
 
 from . import connection
 from . import events
-from . import functools as irc_functools
 from . import features
 from . import ctcp
 from . import message
 from . import schedule
 
+__metaclass__ = type
+
 log = logging.getLogger(__name__)
 
 # set the version tuple
 try:
     VERSION_STRING = pkg_resources.require('irc')[0].version
-    VERSION = tuple(int(res) for res in re.findall('\d+', VERSION_STRING))
+    VERSION = tuple(int(res) for res in re.findall(r'\d+', VERSION_STRING))
 except Exception:
     VERSION_STRING = 'unknown'
     VERSION = ()
@@ -105,269 +107,8 @@ class MessageTooLong(ValueError):
     "Message is too long"
 
 
-class PrioritizedHandler(
-        collections.namedtuple('Base', ('priority', 'callback'))):
-    def __lt__(self, other):
-        "when sorting prioritized handlers, only use the priority"
-        return self.priority < other.priority
-
-
-class Reactor(object):
-    """
-    Processes events from one or more IRC server connections.
-
-    This class implements a reactor in the style of the `reactor pattern
-    <http://en.wikipedia.org/wiki/Reactor_pattern>`_.
-
-    When a Reactor object has been instantiated, it can be used to create
-    Connection objects that represent the IRC connections.  The
-    responsibility of the reactor object is to provide an event-driven
-    framework for the connections and to keep the connections alive.
-    It runs a select loop to poll each connection's TCP socket and
-    hands over the sockets with incoming data for processing by the
-    corresponding connection.
-
-    The methods of most interest for an IRC client writer are server,
-    add_global_handler, remove_global_handler,
-    process_once, and process_forever.
-
-    This is functionally an event-loop which can either use it's own
-    internal polling loop, or tie into an external event-loop, by
-    having the external event-system periodically call `process_once`
-    on the instantiated reactor class. This will allow the reactor
-    to process any queued data and/or events.
-
-    Calling `process_forever` will hand off execution to the reactor's
-    internal event-loop, which will not return for the life of the
-    reactor.
-
-    Here is an example:
-
-        client = irc.client.Reactor()
-        server = client.server()
-        server.connect("irc.some.where", 6667, "my_nickname")
-        server.privmsg("a_nickname", "Hi there!")
-        client.process_forever()
-
-    This will connect to the IRC server irc.some.where on port 6667
-    using the nickname my_nickname and send the message "Hi there!"
-    to the nickname a_nickname.
-
-    The methods of this class are thread-safe; accesses to and modifications
-    of its internal lists of connections, handlers, and delayed commands
-    are guarded by a mutex.
-    """
-
-    scheduler_class = schedule.DefaultScheduler
-
-    def __do_nothing(*args, **kwargs):
-        pass
-
-    def __init__(self, on_connect=__do_nothing, on_disconnect=__do_nothing):
-        """Constructor for Reactor objects.
-
-        on_connect: optional callback invoked when a new connection
-        is made.
-
-        on_disconnect: optional callback invoked when a socket is
-        disconnected.
-
-        The arguments mainly exist to be able to use an external
-        main loop (for example Tkinter's or PyGTK's main app loop)
-        instead of calling the process_forever method.
-
-        An alternative is to just call ServerConnection.process_once()
-        once in a while.
-        """
-
-        self._on_connect = on_connect
-        self._on_disconnect = on_disconnect
-
-        scheduler = self.scheduler_class()
-        assert isinstance(scheduler, schedule.IScheduler)
-        self.scheduler = scheduler
-
-        self.connections = []
-        self.handlers = {}
-        # Modifications to these shared lists and dict need to be thread-safe
-        self.mutex = threading.RLock()
-
-        self.add_global_handler("ping", _ping_ponger, -42)
-
-    def server(self):
-        """Creates and returns a ServerConnection object."""
-
-        c = ServerConnection(self)
-        with self.mutex:
-            self.connections.append(c)
-        return c
-
-    def process_data(self, sockets):
-        """Called when there is more data to read on connection sockets.
-
-        Arguments:
-
-            sockets -- A list of socket objects.
-
-        See documentation for Reactor.__init__.
-        """
-        with self.mutex:
-            log.log(logging.DEBUG-2, "process_data()")
-            for s, c in itertools.product(sockets, self.connections):
-                if s == c.socket:
-                    c.process_data()
-
-    def process_timeout(self):
-        """Called when a timeout notification is due.
-
-        See documentation for Reactor.__init__.
-        """
-        with self.mutex:
-            self.scheduler.run_pending()
-
-    @property
-    def sockets(self):
-        with self.mutex:
-            return [
-                conn.socket
-                for conn in self.connections
-                if conn is not None
-                and conn.socket is not None
-            ]
-
-    def process_once(self, timeout=0):
-        """Process data from connections once.
-
-        Arguments:
-
-            timeout -- How long the select() call should wait if no
-                       data is available.
-
-        This method should be called periodically to check and process
-        incoming data, if there are any.  If that seems boring, look
-        at the process_forever method.
-        """
-        log.log(logging.DEBUG - 2, "process_once()")
-        sockets = self.sockets
-        if sockets:
-            (i, o, e) = select.select(sockets, [], [], timeout)
-            self.process_data(i)
-        else:
-            time.sleep(timeout)
-        self.process_timeout()
-
-    def process_forever(self, timeout=0.2):
-        """Run an infinite loop, processing data from connections.
-
-        This method repeatedly calls process_once.
-
-        Arguments:
-
-            timeout -- Parameter to pass to process_once.
-        """
-        # This loop should specifically *not* be mutex-locked.
-        # Otherwise no other thread would ever be able to change
-        # the shared state of a Reactor object running this function.
-        log.debug("process_forever(timeout=%s)", timeout)
-        one = functools.partial(self.process_once, timeout=timeout)
-        consume(infinite_call(one))
-
-    def disconnect_all(self, message=""):
-        """Disconnects all connections."""
-        with self.mutex:
-            for c in self.connections:
-                c.disconnect(message)
-
-    def add_global_handler(self, event, handler, priority=0):
-        """Adds a global handler function for a specific event type.
-
-        Arguments:
-
-            event -- Event type (a string).  Check the values of
-                     numeric_events for possible event types.
-
-            handler -- Callback function taking 'connection' and 'event'
-                       parameters.
-
-            priority -- A number (the lower number, the higher priority).
-
-        The handler function is called whenever the specified event is
-        triggered in any of the connections.  See documentation for
-        the Event class.
-
-        The handler functions are called in priority order (lowest
-        number is highest priority).  If a handler function returns
-        "NO MORE", no more handlers will be called.
-        """
-        handler = PrioritizedHandler(priority, handler)
-        with self.mutex:
-            event_handlers = self.handlers.setdefault(event, [])
-            bisect.insort(event_handlers, handler)
-
-    def remove_global_handler(self, event, handler):
-        """Removes a global handler function.
-
-        Arguments:
-
-            event -- Event type (a string).
-            handler -- Callback function.
-
-        Returns 1 on success, otherwise 0.
-        """
-        with self.mutex:
-            if event not in self.handlers:
-                return 0
-            for h in self.handlers[event]:
-                if handler == h.callback:
-                    self.handlers[event].remove(h)
-        return 1
-
-    def dcc(self, dcctype="chat"):
-        """Creates and returns a DCCConnection object.
-
-        Arguments:
-
-            dcctype -- "chat" for DCC CHAT connections or "raw" for
-                       DCC SEND (or other DCC types). If "chat",
-                       incoming data will be split in newline-separated
-                       chunks. If "raw", incoming data is not touched.
-        """
-        with self.mutex:
-            c = DCCConnection(self, dcctype)
-            self.connections.append(c)
-        return c
-
-    def _handle_event(self, connection, event):
-        """
-        Handle an Event event incoming on ServerConnection connection.
-        """
-        with self.mutex:
-            h = self.handlers
-            matching_handlers = sorted(
-                h.get("all_events", []) +
-                h.get(event.type, [])
-            )
-            for handler in matching_handlers:
-                result = handler.callback(connection, event)
-                if result == "NO MORE":
-                    return
-
-    def _remove_connection(self, connection):
-        """[Internal]"""
-        with self.mutex:
-            self.connections.remove(connection)
-            self._on_disconnect(connection.socket)
-
-
-_cmd_pat = (
-    "^(@(?P<tags>[^ ]*) )?(:(?P<prefix>[^ ]+) +)?"
-    "(?P<command>[^ ]+)( *(?P<argument> .+))?"
-)
-_rfc_1459_command_regexp = re.compile(_cmd_pat)
-
-
 @six.add_metaclass(abc.ABCMeta)
-class Connection(object):
+class Connection:
     """
     Base class for IRC connections.
     """
@@ -405,14 +146,14 @@ class ServerConnection(Connection):
 
     buffer_class = buffer.DecodingLineBuffer
     socket = None
+    connected = False
 
     def __init__(self, reactor):
         super(ServerConnection, self).__init__(reactor)
-        self.connected = False
         self.features = features.FeatureSet()
 
     # save the method args to allow for easier reconnection.
-    @irc_functools.save_method_args
+    @jaraco.functools.save_method_args
     def connect(
             self, server, port, nickname, password=None, username=None,
             ircname=None, connect_factory=connection.Factory()):
@@ -726,11 +467,11 @@ class ServerConnection(Connection):
 
             message -- Quit message.
         """
-        if not self.connected:
+        try:
+            del self.connected
+        except AttributeError:
             return
 
-        self.connected = 0
-
         self.quit(message)
 
         try:
@@ -941,6 +682,267 @@ class ServerConnection(Connection):
         self.reactor.scheduler.execute_every(period=interval, func=pinger)
 
 
+class PrioritizedHandler(
+        collections.namedtuple('Base', ('priority', 'callback'))):
+    def __lt__(self, other):
+        "when sorting prioritized handlers, only use the priority"
+        return self.priority < other.priority
+
+
+class Reactor:
+    """
+    Processes events from one or more IRC server connections.
+
+    This class implements a reactor in the style of the `reactor pattern
+    <http://en.wikipedia.org/wiki/Reactor_pattern>`_.
+
+    When a Reactor object has been instantiated, it can be used to create
+    Connection objects that represent the IRC connections.  The
+    responsibility of the reactor object is to provide an event-driven
+    framework for the connections and to keep the connections alive.
+    It runs a select loop to poll each connection's TCP socket and
+    hands over the sockets with incoming data for processing by the
+    corresponding connection.
+
+    The methods of most interest for an IRC client writer are server,
+    add_global_handler, remove_global_handler,
+    process_once, and process_forever.
+
+    This is functionally an event-loop which can either use it's own
+    internal polling loop, or tie into an external event-loop, by
+    having the external event-system periodically call `process_once`
+    on the instantiated reactor class. This will allow the reactor
+    to process any queued data and/or events.
+
+    Calling `process_forever` will hand off execution to the reactor's
+    internal event-loop, which will not return for the life of the
+    reactor.
+
+    Here is an example:
+
+        client = irc.client.Reactor()
+        server = client.server()
+        server.connect("irc.some.where", 6667, "my_nickname")
+        server.privmsg("a_nickname", "Hi there!")
+        client.process_forever()
+
+    This will connect to the IRC server irc.some.where on port 6667
+    using the nickname my_nickname and send the message "Hi there!"
+    to the nickname a_nickname.
+
+    The methods of this class are thread-safe; accesses to and modifications
+    of its internal lists of connections, handlers, and delayed commands
+    are guarded by a mutex.
+    """
+
+    scheduler_class = schedule.DefaultScheduler
+    connection_class = ServerConnection
+
+    def __do_nothing(*args, **kwargs):
+        pass
+
+    def __init__(self, on_connect=__do_nothing, on_disconnect=__do_nothing):
+        """Constructor for Reactor objects.
+
+        on_connect: optional callback invoked when a new connection
+        is made.
+
+        on_disconnect: optional callback invoked when a socket is
+        disconnected.
+
+        The arguments mainly exist to be able to use an external
+        main loop (for example Tkinter's or PyGTK's main app loop)
+        instead of calling the process_forever method.
+
+        An alternative is to just call ServerConnection.process_once()
+        once in a while.
+        """
+
+        self._on_connect = on_connect
+        self._on_disconnect = on_disconnect
+
+        scheduler = self.scheduler_class()
+        assert isinstance(scheduler, schedule.IScheduler)
+        self.scheduler = scheduler
+
+        self.connections = []
+        self.handlers = {}
+        # Modifications to these shared lists and dict need to be thread-safe
+        self.mutex = threading.RLock()
+
+        self.add_global_handler("ping", _ping_ponger, -42)
+
+    def server(self):
+        """Creates and returns a ServerConnection object."""
+
+        conn = self.connection_class(self)
+        with self.mutex:
+            self.connections.append(conn)
+        return conn
+
+    def process_data(self, sockets):
+        """Called when there is more data to read on connection sockets.
+
+        Arguments:
+
+            sockets -- A list of socket objects.
+
+        See documentation for Reactor.__init__.
+        """
+        with self.mutex:
+            log.log(logging.DEBUG - 2, "process_data()")
+            for sock, conn in itertools.product(sockets, self.connections):
+                if sock == conn.socket:
+                    conn.process_data()
+
+    def process_timeout(self):
+        """Called when a timeout notification is due.
+
+        See documentation for Reactor.__init__.
+        """
+        with self.mutex:
+            self.scheduler.run_pending()
+
+    @property
+    def sockets(self):
+        with self.mutex:
+            return [
+                conn.socket
+                for conn in self.connections
+                if conn is not None
+                and conn.socket is not None
+            ]
+
+    def process_once(self, timeout=0):
+        """Process data from connections once.
+
+        Arguments:
+
+            timeout -- How long the select() call should wait if no
+                       data is available.
+
+        This method should be called periodically to check and process
+        incoming data, if there are any.  If that seems boring, look
+        at the process_forever method.
+        """
+        log.log(logging.DEBUG - 2, "process_once()")
+        sockets = self.sockets
+        if sockets:
+            in_, out, err = select.select(sockets, [], [], timeout)
+            self.process_data(in_)
+        else:
+            time.sleep(timeout)
+        self.process_timeout()
+
+    def process_forever(self, timeout=0.2):
+        """Run an infinite loop, processing data from connections.
+
+        This method repeatedly calls process_once.
+
+        Arguments:
+
+            timeout -- Parameter to pass to process_once.
+        """
+        # This loop should specifically *not* be mutex-locked.
+        # Otherwise no other thread would ever be able to change
+        # the shared state of a Reactor object running this function.
+        log.debug("process_forever(timeout=%s)", timeout)
+        one = functools.partial(self.process_once, timeout=timeout)
+        consume(infinite_call(one))
+
+    def disconnect_all(self, message=""):
+        """Disconnects all connections."""
+        with self.mutex:
+            for conn in self.connections:
+                conn.disconnect(message)
+
+    def add_global_handler(self, event, handler, priority=0):
+        """Adds a global handler function for a specific event type.
+
+        Arguments:
+
+            event -- Event type (a string).  Check the values of
+                     numeric_events for possible event types.
+
+            handler -- Callback function taking 'connection' and 'event'
+                       parameters.
+
+            priority -- A number (the lower number, the higher priority).
+
+        The handler function is called whenever the specified event is
+        triggered in any of the connections.  See documentation for
+        the Event class.
+
+        The handler functions are called in priority order (lowest
+        number is highest priority).  If a handler function returns
+        "NO MORE", no more handlers will be called.
+        """
+        handler = PrioritizedHandler(priority, handler)
+        with self.mutex:
+            event_handlers = self.handlers.setdefault(event, [])
+            bisect.insort(event_handlers, handler)
+
+    def remove_global_handler(self, event, handler):
+        """Removes a global handler function.
+
+        Arguments:
+
+            event -- Event type (a string).
+            handler -- Callback function.
+
+        Returns 1 on success, otherwise 0.
+        """
+        with self.mutex:
+            if event not in self.handlers:
+                return 0
+            for h in self.handlers[event]:
+                if handler == h.callback:
+                    self.handlers[event].remove(h)
+        return 1
+
+    def dcc(self, dcctype="chat"):
+        """Creates and returns a DCCConnection object.
+
+        Arguments:
+
+            dcctype -- "chat" for DCC CHAT connections or "raw" for
+                       DCC SEND (or other DCC types). If "chat",
+                       incoming data will be split in newline-separated
+                       chunks. If "raw", incoming data is not touched.
+        """
+        with self.mutex:
+            conn = DCCConnection(self, dcctype)
+            self.connections.append(conn)
+        return conn
+
+    def _handle_event(self, connection, event):
+        """
+        Handle an Event event incoming on ServerConnection connection.
+        """
+        with self.mutex:
+            matching_handlers = sorted(
+                self.handlers.get("all_events", [])
+                + self.handlers.get(event.type, [])
+            )
+            for handler in matching_handlers:
+                result = handler.callback(connection, event)
+                if result == "NO MORE":
+                    return
+
+    def _remove_connection(self, connection):
+        """[Internal]"""
+        with self.mutex:
+            self.connections.remove(connection)
+            self._on_disconnect(connection.socket)
+
+
+_cmd_pat = (
+    "^(@(?P<tags>[^ ]*) )?(:(?P<prefix>[^ ]+) +)?"
+    "(?P<command>[^ ]+)( *(?P<argument> .+))?"
+)
+_rfc_1459_command_regexp = re.compile(_cmd_pat)
+
+
 class DCCConnectionError(IRCError):
     pass
 
@@ -953,14 +955,14 @@ class DCCConnection(Connection):
     method on a Reactor object.
     """
     socket = None
+    connected = False
+    passive = False
+    peeraddress = None
+    peerport = None
 
     def __init__(self, reactor, dcctype):
         super(DCCConnection, self).__init__(reactor)
-        self.connected = 0
-        self.passive = 0
         self.dcctype = dcctype
-        self.peeraddress = None
-        self.peerport = None
 
     def connect(self, address, port):
         """Connect/reconnect to a DCC peer.
@@ -977,12 +979,11 @@ class DCCConnection(Connection):
         self.buffer = buffer.LineBuffer()
         self.handlers = {}
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.passive = 0
         try:
             self.socket.connect((self.peeraddress, self.peerport))
         except socket.error as x:
             raise DCCConnectionError("Couldn't connect to socket: %s" % x)
-        self.connected = 1
+        self.connected = True
         self.reactor._on_connect(self.socket)
         return self
 
@@ -999,7 +1000,7 @@ class DCCConnection(Connection):
         self.buffer = buffer.LineBuffer()
         self.handlers = {}
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.passive = 1
+        self.passive = True
         try:
             self.socket.bind((socket.gethostbyname(socket.gethostname()), 0))
             self.localaddress, self.localport = self.socket.getsockname()
@@ -1015,10 +1016,11 @@ class DCCConnection(Connection):
 
             message -- Quit message.
         """
-        if not self.connected:
+        try:
+            del self.connected
+        except AttributeError:
             return
 
-        self.connected = 0
         try:
             self.socket.shutdown(socket.SHUT_WR)
             self.socket.close()
@@ -1037,7 +1039,7 @@ class DCCConnection(Connection):
             conn, (self.peeraddress, self.peerport) = self.socket.accept()
             self.socket.close()
             self.socket = conn
-            self.connected = 1
+            self.connected = True
             log.debug(
                 "DCC connection from %s:%d", self.peeraddress,
                 self.peerport)
@@ -1105,7 +1107,7 @@ class DCCConnection(Connection):
             self.disconnect("Connection reset by peer.")
 
 
-class SimpleIRCClient(object):
+class SimpleIRCClient:
     """A simple single-server IRC client class.
 
     This is an example of an object-oriented wrapper of the IRC
@@ -1153,12 +1155,13 @@ class SimpleIRCClient(object):
         """
         log.debug("_dispatcher: %s", event.type)
 
-        do_nothing = lambda c, e: None
+        def do_nothing(connection, event):
+            return None
         method = getattr(self, "on_" + event.type, do_nothing)
         method(connection, event)
 
-    def _dcc_disconnect(self, c, e):
-        self.dcc_connections.remove(c)
+    def _dcc_disconnect(self, connection, event):
+        self.dcc_connections.remove(connection)
 
     def connect(self, *args, **kwargs):
         """Connect using the underlying connection"""
@@ -1195,7 +1198,7 @@ class SimpleIRCClient(object):
         self.reactor.process_forever()
 
 
-class Event(object):
+class Event:
     """
     An IRC event.
 
@@ -1255,8 +1258,7 @@ def ip_numstr_to_quad(num):
     >>> ip_numstr_to_quad(3232235521)
     '192.168.0.1'
     """
-    n = int(num)
-    packed = struct.pack('>L', n)
+    packed = struct.pack('>L', int(num))
     bytes = struct.unpack('BBBB', packed)
     return ".".join(map(str, bytes))
 
