@@ -2,9 +2,14 @@ from __future__ import absolute_import
 
 import socket
 
-identity = lambda x: x
+__metaclass__ = type
 
-class Factory(object):
+
+def identity(x):
+    return x
+
+
+class Factory:
     """
     A class for creating custom socket connections.
 
